/*
* Copyright 2014 Freescale Semiconductor, Inc.
* Copyright 2020, 2023 NXP
*
* SPDX-License-Identifier: BSD-3-Clause
*/

/**
 @file
 @brief AVDECC static configuration
 @details Contains all compile time configuration options for avdecc
*/

#ifndef _AVDECC_CFG_H_
#define _AVDECC_CFG_H_

#include "common/config.h"
#include "common/adp.h"

#define avdecc_CFG_LOG	CFG_LOG

#define CFG_AVDECC_MAX_TIMERS_PER_ENTITY	6

#define AVDECC_CFG_INFLIGHT_TIMER_RESOLUTION	10

#define AVDECC_CFG_ENTITY_LOCK_TIMER_GRANULARITY_MS	100
#define AVDECC_CFG_ENTITY_LOCK_TIMER_MS			(1000 * 60) /* 1 min entity lock timer. */

#define AECP_CFG_MAX_AEM_IN_PROGRESS		(10000 / AECP_IN_PROGRESS_TIMEOUT) /* 10000 ms : Maximum IN_PROGRESS responses for AECP CMD before declaring the application unresponsive*/

#define CFG_AECP_DEFAULT_NUM_UNSOLICITED		8
#define CFG_AECP_MAX_NUM_UNSOLICITED			64
#define CFG_AECP_MIN_NUM_UNSOLICITED			1

#define CFG_ADP_DEFAULT_NUM_ENTITIES_DISCOVERY		16
#define CFG_ADP_MIN_NUM_ENTITIES_DISCOVERY		8
#define CFG_ADP_MAX_NUM_ENTITIES_DISCOVERY		128

#define CFG_DEFAULT_AVB_INTERFACE_INDEX		0

#define CFG_AVDECC_DEFAULT_NUM_INFLIGHTS		5
#define CFG_AVDECC_MAX_NUM_INFLIGHTS			(CFG_ADP_MAX_NUM_ENTITIES_DISCOVERY)
#define CFG_AVDECC_MIN_NUM_INFLIGHTS			5

#define CFG_ADP_DEFAULT_VALID_TIME	62 //seconds
#define CFG_ADP_MIN_VALID_TIME		2 //seconds
#define CFG_ADP_MAX_VALID_TIME		62 //seconds
#define CFG_ADP_MILAN_VALID_TIME	20 //seconds  /* AVNU.IO.CONTROL (9.2) */

#define CFG_ACMP_DEFAULT_NUM_TALKER_STREAMS	8
#define CFG_ACMP_MIN_NUM_TALKER_STREAMS		1
#define CFG_ACMP_MAX_NUM_TALKER_STREAMS		32

#define CFG_ACMP_DEFAULT_NUM_LISTENER_STREAMS	8
#define CFG_ACMP_MIN_NUM_LISTENER_STREAMS	1
#define CFG_ACMP_MAX_NUM_LISTENER_STREAMS	32

#define CFG_ACMP_DEFAULT_NUM_LISTENER_PAIRS	10
#define CFG_ACMP_MIN_NUM_LISTENER_PAIRS		1
#define CFG_ACMP_MAX_NUM_LISTENER_PAIRS		512

#define CFG_ACMP_DEFAULT_MAAP_BASE_RANGE_ID	1000 /* MAAP base range id for ACMP talkers */
#define CFG_ACMP_DEFAULT_MAAP_COUNT_PER_RANGE	1    /* One MAAP range per ACMP talker */

#endif /* _AVDECC_CFG_H_ */
