if(NOT DEFINED CONFIG_DIR)
  message(FATAL_ERROR "CONFIG_DIR not defined")
endif()

function(system_cfg_install src_system_cfg dst_system_cfg)
  if(EXISTS ${src_system_cfg})
    install(FILES ${src_system_cfg} DESTINATION ${CONFIG_DIR} PERMISSIONS OWNER_WRITE OWNER_READ GROUP_WRITE GROUP_READ RENAME ${dst_system_cfg})
  endif()
endfunction()

function(srp_cfg_install name)
  set(target_srp_cfg_file ${CMAKE_CURRENT_LIST_DIR}/${TARGET}_${CONFIG}_srp${name}.cfg)
  if(EXISTS ${target_srp_cfg_file})
    install(FILES ${target_srp_cfg_file} DESTINATION ${CONFIG_DIR} PERMISSIONS OWNER_WRITE OWNER_READ GROUP_WRITE GROUP_READ RENAME srp${name}.cfg)
  else()
    install(FILES ${CMAKE_CURRENT_LIST_DIR}/srp${name}.cfg DESTINATION ${CONFIG_DIR} PERMISSIONS OWNER_WRITE OWNER_READ GROUP_WRITE GROUP_READ)
  endif()
endfunction()

if(${CONFIG} STREQUAL "endpoint_avb")
  install(DIRECTORY DESTINATION ${CONFIG_DIR})
  install(FILES ${CMAKE_CURRENT_LIST_DIR}/config DESTINATION ${CONFIG_DIR} PERMISSIONS OWNER_WRITE OWNER_READ GROUP_WRITE GROUP_READ)
  install(FILES ${CMAKE_CURRENT_LIST_DIR}/config_avb DESTINATION ${CONFIG_DIR} PERMISSIONS OWNER_WRITE OWNER_READ GROUP_WRITE GROUP_READ)
  # remove the second config
  install(CODE "execute_process(COMMAND sed -i \"/.*GENAVB_TSN_CONFIG.*2/d\" \$ENV{DESTDIR}/${CMAKE_INSTALL_PREFIX}/${CONFIG_DIR}/config)")
  # set AVB endpoint as only and first config
  install(CODE "execute_process(COMMAND sed -i \"s/Endpoint TSN/Endpoint AVB/g\" \$ENV{DESTDIR}/${CMAKE_INSTALL_PREFIX}/${CONFIG_DIR}/config)")
  install(CODE "execute_process(COMMAND sed -i \"s/config_tsn/config_avb/g\" \$ENV{DESTDIR}/${CMAKE_INSTALL_PREFIX}/${CONFIG_DIR}/config)")
  install(DIRECTORY ${CMAKE_CURRENT_LIST_DIR}/ DESTINATION ${CONFIG_DIR} FILE_PERMISSIONS OWNER_WRITE OWNER_READ GROUP_WRITE GROUP_READ FILES_MATCHING PATTERN "apps*.cfg")
  install(DIRECTORY ${CMAKE_CURRENT_LIST_DIR}/ DESTINATION ${CONFIG_DIR} FILE_PERMISSIONS OWNER_WRITE OWNER_READ GROUP_WRITE GROUP_READ FILES_MATCHING PATTERN "genavb*.cfg")
  install(DIRECTORY ${CMAKE_CURRENT_LIST_DIR}/ DESTINATION ${CONFIG_DIR} FILE_PERMISSIONS OWNER_WRITE OWNER_READ GROUP_WRITE GROUP_READ FILES_MATCHING PATTERN "fgptp.cfg*")
  srp_cfg_install("")
  system_cfg_install("${CMAKE_CURRENT_LIST_DIR}/${TARGET}_${CONFIG}_system.cfg" "system.cfg")
elseif(${CONFIG} STREQUAL "endpoint_tsn")
  install(DIRECTORY DESTINATION ${CONFIG_DIR})
  install(FILES ${CMAKE_CURRENT_LIST_DIR}/config DESTINATION ${CONFIG_DIR} PERMISSIONS OWNER_WRITE OWNER_READ GROUP_WRITE GROUP_READ)
  # remove the second config and keep TSN as the first and only config
  install(CODE "execute_process(COMMAND sed -i \"/.*GENAVB_TSN_CONFIG.*2/d\" \$ENV{DESTDIR}/${CMAKE_INSTALL_PREFIX}/${CONFIG_DIR}/config)")
  install(FILES ${CMAKE_CURRENT_LIST_DIR}/config_tsn DESTINATION ${CONFIG_DIR} PERMISSIONS OWNER_WRITE OWNER_READ GROUP_WRITE GROUP_READ)
  install(DIRECTORY ${CMAKE_CURRENT_LIST_DIR}/ DESTINATION ${CONFIG_DIR} FILE_PERMISSIONS OWNER_WRITE OWNER_READ GROUP_WRITE GROUP_READ FILES_MATCHING PATTERN "apps-tsn*.cfg")
  install(DIRECTORY ${CMAKE_CURRENT_LIST_DIR}/ DESTINATION ${CONFIG_DIR} FILE_PERMISSIONS OWNER_WRITE OWNER_READ GROUP_WRITE GROUP_READ FILES_MATCHING PATTERN "fgptp.cfg*")
  srp_cfg_install("")
  system_cfg_install("${CMAKE_CURRENT_LIST_DIR}/${TARGET}_${CONFIG}_system.cfg" "system.cfg")
elseif(${CONFIG} STREQUAL "endpoint_avb_tsn")
  install(DIRECTORY DESTINATION ${CONFIG_DIR})
  install(FILES ${CMAKE_CURRENT_LIST_DIR}/config DESTINATION ${CONFIG_DIR} PERMISSIONS OWNER_WRITE OWNER_READ GROUP_WRITE GROUP_READ)
  install(FILES ${CMAKE_CURRENT_LIST_DIR}/config_tsn DESTINATION ${CONFIG_DIR} PERMISSIONS OWNER_WRITE OWNER_READ GROUP_WRITE GROUP_READ)
  install(FILES ${CMAKE_CURRENT_LIST_DIR}/config_avb DESTINATION ${CONFIG_DIR} PERMISSIONS OWNER_WRITE OWNER_READ GROUP_WRITE GROUP_READ)
  install(DIRECTORY ${CMAKE_CURRENT_LIST_DIR}/ DESTINATION ${CONFIG_DIR} FILE_PERMISSIONS OWNER_WRITE OWNER_READ GROUP_WRITE GROUP_READ FILES_MATCHING PATTERN "apps*.cfg")
  install(DIRECTORY ${CMAKE_CURRENT_LIST_DIR}/ DESTINATION ${CONFIG_DIR} FILE_PERMISSIONS OWNER_WRITE OWNER_READ GROUP_WRITE GROUP_READ FILES_MATCHING PATTERN "genavb*.cfg")
  install(DIRECTORY ${CMAKE_CURRENT_LIST_DIR}/ DESTINATION ${CONFIG_DIR} FILE_PERMISSIONS OWNER_WRITE OWNER_READ GROUP_WRITE GROUP_READ FILES_MATCHING PATTERN "fgptp.cfg*")
  srp_cfg_install("")
  system_cfg_install("${CMAKE_CURRENT_LIST_DIR}/${TARGET}_endpoint_avb_system.cfg" "system.cfg.avb")
  system_cfg_install("${CMAKE_CURRENT_LIST_DIR}/${TARGET}_endpoint_tsn_system.cfg" "system.cfg.tsn")
elseif(${CONFIG} STREQUAL "bridge")
  install(DIRECTORY DESTINATION ${CONFIG_DIR})
  install(FILES ${CMAKE_CURRENT_LIST_DIR}/config DESTINATION ${CONFIG_DIR} PERMISSIONS OWNER_WRITE OWNER_READ GROUP_WRITE GROUP_READ)
  # Keep only the auto start config
  install(CODE "execute_process(COMMAND sed -i \"1,/CFG_AUTO_START/!d\" \$ENV{DESTDIR}/${CMAKE_INSTALL_PREFIX}/${CONFIG_DIR}/config)")
  install(DIRECTORY ${CMAKE_CURRENT_LIST_DIR}/ DESTINATION ${CONFIG_DIR} FILE_PERMISSIONS OWNER_WRITE OWNER_READ GROUP_WRITE GROUP_READ FILES_MATCHING PATTERN "fgptp-br.cfg*")
  srp_cfg_install("-br")
  system_cfg_install("${CMAKE_CURRENT_LIST_DIR}/${TARGET}_${CONFIG}_system.cfg" "system.cfg")
endif()
